/*
 *  popup.c  - Handles the loading, parsing, and filtering of popups.
 *
 *  Added by Steve Kemp <skx@tardis.ed.ac.uk>
 *
 * Distributed under the GNU General Public License; see the README file.
 * This code comes with NO WARRANTY. http://www.junkbusters.com/ht/en/gpl.html
 *
 *  The code prefixed by a CHANGED comment was written/modified by:
 *   James Greenhalgh <james@xenon.office.ptel.co.uk>
 *
 *  
 *  popup.c  - Handles the loading, parsing, and filtering of popups.
 * $Id: popup.c,v 1.2 2000/09/17 06:58:33 swa Exp $
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <string.h>
#include <malloc.h>
#include <errno.h>
#include <sys/stat.h>
#include <ctype.h>

#ifndef _WIN32
#include <unistd.h>
#endif

#ifdef REGEX
#include <gnu_regex.h>
#endif

#include "jcc.h"

/* Protypes. */
void unload_popupfile(struct client_state *csp /* Not used */ );

/* Globals */
static struct popup_blocklist *blocked = NULL;
static struct popup_blocklist *allowed = NULL;
extern int kill_all_popups;

/* Change these for debug output.  *lots*. */
/*#define POPUP_VERBOSE 1*/
/* CHANGED - added the below and shifted the more spammy stuff into it ;-) */
#undef POPUP_VERY_VERBOSE
#undef POPUP_VERBOSE


/* Load, and parse the blocklist. */
int
load_popupfile(struct client_state *csp)
{
  FILE *fp;
  char  buf[BUFSIZ], *p, *q;
  struct popup_blocklist *entry = NULL;
  struct popup_blocklist *temp  = NULL;
  p = buf;
  q = buf;

  /* Unload the previous popup file, if any. */
  unload_popupfile( csp );

  if((fp = fopen(popupfile, "r")) == NULL)
  {
      printf( "** Failed to open popupfile: %s.\n", popupfile);
      return -1;
  }

  while(fgets(buf, sizeof(buf), fp)) 
  {
      /* comments */
      if((p = strchr(buf, '#')))
          *p = '\0';

      /* elide white-space */
      for(p = q = buf; *q ; q++)
      {
          if(!isspace((int)*q))
          {
              *p++ = *q;
          }
      }
      
      /* linefeeds */
      if((p = strpbrk(buf, "\r\n")) != NULL)
      {
          *p = '\0';
      }

      /* we've reduced the whole line..*/
      if ( !strlen( buf ) )
      {
          continue;
      }

      /* Handle allowed hosts. */
      if ( *buf == '~' )
      {
          /* Rememeber: skip the tilde */
          entry = (struct popup_blocklist*)malloc( sizeof( struct popup_blocklist ) );
          entry->host_name = strdup( buf + 1 );
          entry->next      = 0;

          if ( allowed == NULL )
          {
              allowed       = entry;
          }
          else
          {
              /* Save current head, replace it with new entry,
              	ad d old list to end of head */
              temp = allowed;
              allowed = entry;
              allowed->next = temp;
          }
      }
      else
      {
#ifdef POPUP_VERBOSE
          printf( "Adding blocked popup site : %s\n", buf );
#endif    

          entry = (struct popup_blocklist*)malloc( sizeof( struct popup_blocklist ) );
          entry->host_name = strdup( buf );
          if ( blocked == NULL )
          {
              blocked       = entry;
              blocked->next = 0;
          }
          else
          {
              entry->next = NULL;
              
              // Save current head, replace it with new entry,
              // add old list to end of head
              temp = blocked;
              blocked = entry;
              blocked->next = temp;
          }
      }
  }
  
  fclose( fp );
  return(0);
}

/* Free the lists of blocked, and allowed popup sites. */
void
unload_popupfile(struct client_state *csp /* Not used */ )
{
    struct popup_blocklist * cur = NULL;
    struct popup_blocklist * temp= NULL;

    /*     Free the blocked list. */
    if ( blocked != NULL )
    {
        cur = blocked;
        
        while( cur->next != NULL )
        {
            temp = cur;
            cur  = cur->next;
            
            free( temp->host_name );
            free( temp );
        }
        blocked = NULL;
    }

    /*    Free the allowed list. */
    if ( allowed != NULL )
    {
        cur = allowed;
        
        while( cur->next != NULL )
        {
            temp = cur;
            cur  = cur->next;
            
            free( temp->host_name );
            free( temp );
        }
        allowed = NULL;
    }
}


/* Filter the block of data that's been read from the server.
   IF NECESSARY.  */
void filter_popups( char *buff, int size, char *host_name )
{
    struct popup_blocklist * cur;
    int i;
    int found = 0;
    char *popup = NULL;
    char *close = NULL;
    char *p     = NULL;
		char *q			= NULL; /* by BREITENB NEW! */

    /* If the hostname is on our list for blocking then mark it
       as a host to   block from.  (This may be later changed if the
       host is also on t  he list-to-allow list).
    */

		buff[size]='\0'; /* terminate with zero at the end! NEW! (see evil hack in jcc.c!) */
		if (size>100) {
			for (i=0; i <50; i++) { /* avoid scanning binary data! */
				 if ((unsigned int)(buff[i])>127) {
#ifdef  POPUP_VERBOSE
		      printf( "I'm not scanning binary stuff! (%i)\n",buff[i]);
#endif  
				 	return;
				}
			}
		}
		
		
    for(cur = blocked ; cur ; cur = cur->next) 
    {
      if ( host_name != 0 )
	{
	  if ( strcmp( cur->host_name, host_name ) == 0 )
	    {
#ifdef  POPUP_VERBOSE
	      printf( "Blocking %s\n", host_name );
#endif  
	      found = 1;
	    }
	}
    }

    /* Force match if we're supposed to nuke _all_ popups, globally. */
    if ( kill_all_popups != 0 )
    {
#ifdef POPUP_VERBOSE    
        printf ("Indescriminatly nuking popups..\n" );
#endif
        found = 1;
    }
		/* an exception-from blocking should still be an exception! by BREITENB NEW! */
    

    /*    Now, if its allowed adjust the filtering, so it _doesn't_ happen. */
    for(cur = allowed ; cur ; cur = cur->next) 
    {
      if ( host_name != 0 )
	{
	  if ( strcmp( cur->host_name, host_name ) == 0 )
	    {
#ifdef POPUP_VERBOSE    
	      printf( "Reversing block decision for %s\n", host_name );
#endif  
	      found = 0;
	    }
	}
    }

    if ( found == 0)
    {
#ifdef POPUP_VERBOSE    
        printf( "Allowing %s\n", host_name );
#endif  
        return;
    }
  
    while (popup = strstr( buff, "window.open(" )) 
    /* if ( popup  )  by BREITENB filter ALL popups! NEW! */
    {
#ifdef POPUP_VERBOSE    
        printf( "Found start of window open" );
#endif  
        close = strstr( popup+1, ");" );
        if ( close )
        {
#ifdef POPUP_VERBOSE
            printf( "Found end of window open" );
#endif  
            for( p = popup; p != (close+1); p++ )
            {
                *p = ' ';
            }
#ifdef POPUP_VERBOSE        
            printf( "Blocked %s\n", host_name );
#endif  
        }
        else
        {
#ifdef POPUP_VERBOSE
            printf( "Couldn't find end, turned into comment.  Read boundary?\n" );
#endif  
            *popup = '/';
            popup++;
            *popup = '/';
        }
				
				
				q=popup; /* by BREITENB NEW! */
				while (q>=buff) {
				  if (*q==' ' || *q=='\t')
						q--;
					else break;
				}
				if (q>=buff) {
					if (*q=='=') *++q='1';
					/* result of popup is assigned to a variable! ensure success. hehehe. */
				}
    }

		/* Filter all other crap like onUnload onExit etc.  (by BREITENB) NEW!*/
		popup=strstr( buff, "<body");
		if (!popup) popup=strstr( buff, "<BODY");
		if (!popup) popup=strstr( buff, "<Body");
		if (!popup) popup=strstr( buff, "<BOdy");
		if (popup) {
			q=strchr(popup,'>');
			if (q) {
				/* we are now between <body and the ending > */
				p=strstr(popup, "onUnload");
				if (p) {
					strncpy(p,"_nU_",4);
				}
				p=strstr(popup, "onExit");
				if (p) {
					strncpy(p,"_nE_",4);
				}
			}
		}
}
