/* Functions for creating, setting and destroying the workspace tray icon
 *
 * Written and copyright 1999 Adam Lock <locka@iol.ie>
 *
 * Distributed under the GNU General Public License; see the README file.
 * This code comes with NO WARRANTY.
 *
 * $Id: w32rulesdlg.c,v 1.1 2000/09/16 15:08:50 swa Exp $
 *
 */
#include <stdio.h>

#include <windows.h>
#include <commctrl.h>

#include "w32res.h"

extern HINSTANCE g_hInstance;

static BOOL CALLBACK DialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);

static HIMAGELIST g_hImageList = NULL;
static char *g_pszDefaultRule;
static BOOL g_bDirty = FALSE;

const int nSmallIconWidth = 16;
const int nSmallIconHeight = 16;

int ShowRulesDialog(HWND hwndParent)
{
	DialogBox(g_hInstance, MAKEINTRESOURCE(IDD_RULES), hwndParent, DialogProc);
	return TRUE;
}

void SetDefaultRule(const char *pszRule)
{
	if (pszRule == NULL)
	{
		if (g_pszDefaultRule)
		{
			free(g_pszDefaultRule);
			g_pszDefaultRule = NULL;
		}
	}
	else
	{
		g_pszDefaultRule = strdup(pszRule);
	}
}

#define IMAGE_ALLOW 0
#define IMAGE_DENY  1

static void InsertRule(HWND hwndListView, const char *pszRule, BOOL bAllow)
{
	LVITEM item;
	item.mask = LVIF_TEXT | LVIF_IMAGE;
	item.pszText = pszRule;
	item.iItem = ListView_GetItemCount(hwndListView) + 1;
	item.iSubItem = 0;
	item.iImage = bAllow ? IMAGE_ALLOW : IMAGE_DENY;
	ListView_InsertItem(hwndListView, &item);
	/* TODO add subitem for whether the rule is always or never */
}


static void SetDirty(BOOL bDirty)
{
	g_bDirty = bDirty;
	/* TODO Change some values */
}


static void OnInitDialog(HWND hwndDlg)
{
	LVCOLUMN aCols[2];
	HWND hwndListView;
	RECT rcListView;
	int cx;

	if (g_hImageList == NULL)
	{
		/* Create image list and add icons */
		HICON hIconDeny = LoadImage(g_hInstance, MAKEINTRESOURCE(IDI_DENYRULE), IMAGE_ICON, nSmallIconWidth, nSmallIconHeight, 0);
		HICON hIconAllow = LoadImage(g_hInstance, MAKEINTRESOURCE(IDI_ALLOWRULE), IMAGE_ICON, nSmallIconWidth, nSmallIconHeight, 0);
		g_hImageList = ImageList_Create(nSmallIconWidth, nSmallIconHeight, ILC_COLOR | ILC_MASK, 0, 10);
		ImageList_AddIcon(g_hImageList, hIconAllow);
		ImageList_AddIcon(g_hImageList, hIconDeny);
	}

	/* Set the default rule value if there is one */
	if (g_pszDefaultRule)
	{
		SetDlgItemText(hwndDlg, IDC_NEW, g_pszDefaultRule);
		SetDefaultRule(NULL);
	}

	/* Initialise the list view */
	hwndListView = GetDlgItem(hwndDlg, IDC_RULES);
	ListView_SetImageList(hwndListView, g_hImageList, LVSIL_SMALL);
	GetClientRect(hwndListView, &rcListView);
	cx = rcListView.right - rcListView.left;
	aCols[0].mask = LVCF_TEXT | LVCF_WIDTH;
	aCols[0].pszText = "Rule";
	aCols[0].cx = (70 * cx) / 100;
	aCols[1].mask = LVCF_TEXT | LVCF_WIDTH;
	aCols[1].pszText = "Applies when";
	aCols[1].cx = cx - aCols[0].cx;
	ListView_InsertColumn(hwndListView, 0, &aCols[0]);
	ListView_InsertColumn(hwndListView, 1, &aCols[1]);

	/* Read and add rules to the list */
	/* TODO */
	InsertRule(hwndListView, "Test rule 1", TRUE);
	InsertRule(hwndListView, "Test rule 2", TRUE);
	InsertRule(hwndListView, "Test rule 3", FALSE);
	InsertRule(hwndListView, "Test rule 4", FALSE);
}


static int GetFirstSelectedItem(HWND hwndDlg)
{
	/* Check for selected items */
	HWND hwndListView = GetDlgItem(hwndDlg, IDC_RULES);
	int nItem = -1;
	do {
		nItem = ListView_GetNextItem(hwndListView, nItem, LVNI_SELECTED);
		if (nItem >= 0)
		{
			return nItem;
		}
	} while (nItem >= 0);
	return -1;
}


static void OnRulesItemChanged(HWND hwndDlg)
{
	int nItem = GetFirstSelectedItem(hwndDlg);
	HWND hwndListView = GetDlgItem(hwndDlg, IDC_RULES);
	int nItems = ListView_GetItemCount(hwndListView);
	BOOL bHaveSelection = (nItem >= 0) ? TRUE : FALSE;
	BOOL bMoveUp = (bHaveSelection && nItem > 0) ? TRUE : FALSE;
	BOOL bMoveDown = (bHaveSelection && nItem < nItems - 1) ? TRUE : FALSE;

	/* Enable/disable buttons */
	EnableWindow(GetDlgItem(hwndDlg, IDC_DELETE), bHaveSelection);
	EnableWindow(GetDlgItem(hwndDlg, IDC_MOVEUP), bMoveUp);
	EnableWindow(GetDlgItem(hwndDlg, IDC_MOVEDOWN), bMoveDown);
}


static void MoveRules(HWND hwndDlg, BOOL bMoveUp)
{
}


static void OnMoveRuleUpClick(HWND hwndDlg)
{
}


static void OnMoveRuleDownClick(HWND hwndDlg)
{
}


static void OnCreateRuleClick(HWND hwndDlg)
{
}


static void OnDeleteRuleClick(HWND hwndDlg)
{
	/* Get selection and remove it */
	int nItem = GetFirstSelectedItem(hwndDlg);
	if (nItem >= 0)
	{
		LVITEM item;
		HWND hwndListView = GetDlgItem(hwndDlg, IDC_RULES);
		item.mask = LVIF_PARAM;
		item.iItem = nItem;
		item.iSubItem = 0;
		ListView_GetItem(hwndListView, &item);
		/* TODO erase data stored with item */
		ListView_DeleteItem(hwndListView, nItem);
	}
}


static void OnCommand(HWND hwndDlg, int nCommand, int nNotifyCode, HWND hwndItem)
{
	switch (nCommand) 
	{ 
	case IDCANCEL:
	case IDC_SAVE:
		EndDialog(hwndDlg, IDOK);
		break;
	case IDC_CREATE:
		if (nNotifyCode == BN_CLICKED)
		{
			OnCreateRuleClick(hwndDlg);
		}
		break;
	case IDC_DELETE:
		if (nNotifyCode == BN_CLICKED)
		{
			OnDeleteRuleClick(hwndDlg);
		}
		break;
	case IDC_MOVEUP:
		if (nNotifyCode == BN_CLICKED)
		{
			OnMoveRuleUpClick(hwndDlg);
		}
		break;
	case IDC_MOVEDOWN:
		if (nNotifyCode == BN_CLICKED)
		{
			OnMoveRuleDownClick(hwndDlg);
		}
		break;
	}
}


static void OnNotify(HWND hwndDlg, int nIdCtrl, LPNMHDR pnmh)
{
	switch (nIdCtrl)
	{
	case IDC_RULES:
		switch (pnmh->code)
		{
		case LVN_ITEMCHANGED:
			OnRulesItemChanged(hwndDlg);
			break;
		}
		break;
	}
}


static void OnDestroy(HWND hwndDlg)
{
	/* TODO any destruction cleanup */
}


static BOOL CALLBACK DialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_INITDIALOG:
		OnInitDialog(hwndDlg);
		return TRUE;
	
	case WM_DESTROY:
		OnDestroy(hwndDlg);
		return TRUE;

	case WM_COMMAND:
		OnCommand(hwndDlg, LOWORD(wParam), HIWORD(wParam), (HWND) lParam);
		break;

	case WM_NOTIFY:
		OnNotify(hwndDlg, (int) wParam, (LPNMHDR) lParam);
		break;
	}
	return FALSE;
}


 
